@@ -1,6 +1,7 @@
 package Pcore::API::Bitbucket;
 
 use Pcore -class;
+use Pcore::API::Bitbucket::Issue;
 
 has account_name => ( is => 'ro', isa => Str, required => 1 );
 has repo_slug    => ( is => 'ro', isa => Str, required => 1 );
@@ -15,16 +16,15 @@ sub _build_auth ($self) {
 }
 
 sub issues ( $self, @ ) {
-    state $init = !!require Pcore::API::Bitbucket::Issue;
-
     my $cb = $_[-1];
 
     # https://confluence.atlassian.com/bitbucket/issues-resource-296095191.html#issuesResource-GETalistofissuesinarepository%27stracker
     my %args = (
-        id      => undef,
-        sort    => 'priority',    # priority, kind, version, component, milestone
-        status  => undef,
-        version => undef,
+        limit     => 50,
+        id        => undef,
+        sort      => 'priority',    # priority, kind, version, component, milestone
+        status    => undef,
+        milestone => undef,
         splice @_, 1, -1,
     );
 
@@ -105,6 +105,40 @@ sub create_version ( $self, $ver, $cb ) {
     return;
 }
 
+sub create_milestone ( $self, $milestone, $cb ) {
+    my $url = "https://api.bitbucket.org/1.0/repositories/@{[$self->account_name]}/@{[$self->repo_slug]}/issues/milestones";
+
+    P->http->post(    #
+        $url,
+        headers => {
+            AUTHORIZATION => $self->auth,
+            CONTENT_TYPE  => 'application/x-www-form-urlencoded; charset=UTF-8',
+        },
+        body      => P->data->to_uri( { name => $milestone } ),
+        on_finish => sub ($res) {
+            my $id;
+
+            $id = P->data->from_json( $res->body )->{id} if $res->status == 200;
+
+            $cb->($id);
+
+            return;
+        },
+    );
+
+    return;
+}
+
+sub set_issue_status ( $self, $id, $status, $cb ) {
+    my $issue = Pcore::API::Bitbucket::Issue->new( { api => $self } );
+
+    $issue->{local_id} = $id;
+
+    $issue->set_status( $status, $cb );
+
+    return;
+}
+
 1;
 ## -----SOURCE FILTER LOG BEGIN-----
 ##
