@@ -1,6 +1,7 @@
 package Pcore::Src::File;
 
-use Pcore qw[-class];
+use Pcore -class;
+use Pcore::Util::Text qw[encode_utf8 decode_eol lcut_all rcut_all rtrim_multi remove_bom];
 use Term::ANSIColor qw[:constants];
 
 has action => ( is => 'ro', isa => Enum [qw[decompress compress obfuscate]], required => 1 );
@@ -27,7 +28,7 @@ has dist_cfg => ( is => 'lazy', isa => HashRef, init_arg => undef );
 
 # CLASS METHODS
 sub cfg ($self) {
-    state $cfg = P->cfg->load( $PROC->res->get('/data/src.perl') );
+    state $cfg = P->cfg->load( $ENV->res->get('/data/src.perl') );
 
     return $cfg;
 }
@@ -160,7 +161,7 @@ sub _build_out_buffer ($self) {
     }
 
     if ( $self->decode ) {
-        require Encode::Guess;
+        state $init = !!require Encode::Guess;
 
         # detect buffer encoding
         my $decoder = Encode::Guess::guess_encoding($buffer);
@@ -186,9 +187,9 @@ sub _build_out_buffer ($self) {
 
         return $self->in_buffer if $@;
 
-        P->file->remove_bom($buffer);
+        remove_bom $buffer;
 
-        P->text->encode_utf8($buffer);
+        encode_utf8 $buffer;
     }
 
     # detect filetype, require and run filter
@@ -205,15 +206,15 @@ sub _build_out_buffer ($self) {
     if ( $self->action eq 'decompress' ) {
 
         # clean buffer
-        P->text->decode_eol($buffer);    # decode CRLF to internal \n representation
+        decode_eol $buffer;    # decode CRLF to internal \n representation
 
-        P->text->lcut_all($buffer);      # trim leading horizontal whitespaces
+        lcut_all $buffer;      # trim leading horizontal whitespaces
 
-        P->text->rcut_all($buffer);      # trim trailing horizontal whitespaces
+        rcut_all $buffer;      # trim trailing horizontal whitespaces
 
-        P->text->rtrim_multi($buffer);   # right trim each line
+        rtrim_multi $buffer;   # right trim each line
 
-        $buffer =~ s/\t/    /smg;        # convert tabs to spaces
+        $buffer =~ s/\t/    /smg;    # convert tabs to spaces
 
         $buffer .= $LF;
     }
@@ -240,6 +241,10 @@ sub run ($self) {
 
     # write file, if it was physically read from disk
     if ( $self->_can_write && !$self->dry_run && $self->was_changed ) {
+
+        # remove READ-ONLY attr under windows
+        chmod 0777, $self->path or 1 if $MSWIN;
+
         P->file->write_bin( $self->path, $self->out_buffer );
     }
 
@@ -253,7 +258,7 @@ sub run ($self) {
 ## ┌──────┬──────────────────────┬────────────────────────────────────────────────────────────────────────────────────────────────────────────────┐
 ## │ Sev. │ Lines                │ Policy                                                                                                         │
 ## ╞══════╪══════════════════════╪════════════════════════════════════════════════════════════════════════════════════════════════════════════════╡
-## │    3 │ 18, 202, 225         │ References::ProhibitDoubleSigils - Double-sigil dereference                                                    │
+## │    3 │ 19, 203, 226         │ References::ProhibitDoubleSigils - Double-sigil dereference                                                    │
 ## └──────┴──────────────────────┴────────────────────────────────────────────────────────────────────────────────────────────────────────────────┘
 ##
 ## -----SOURCE FILTER LOG END-----
